@@ -15,7 +15,7 @@ var commandDefintion = &cobra.Command{
 	Short: `Make the path if it doesn't already exist.`,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
-		fdst := cmd.NewFsDst(args)
+		fdst := cmd.NewFsDir(args)
 		cmd.Run(true, false, command, func() error {
 			return operations.Mkdir(fdst, "")
 		})
