@@ -20,7 +20,7 @@ you want to selectively delete files.
 `,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
-		fdst := cmd.NewFsDst(args)
+		fdst := cmd.NewFsDir(args)
 		cmd.Run(true, false, command, func() error {
 			return operations.Purge(fdst, "")
 		})
