@@ -2,7 +2,7 @@ package rmdir
 
 import (
 	"github.com/ncw/rclone/cmd"
-	"github.com/ncw/rclone/fs"
+	"github.com/ncw/rclone/fs/operations"
 	"github.com/spf13/cobra"
 )
 
@@ -18,9 +18,9 @@ Remove the path.  Note that you can't remove a path with
 objects in it, use purge for that.`,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
-		fdst := cmd.NewFsDst(args)
+		fdst := cmd.NewFsDir(args)
 		cmd.Run(true, false, command, func() error {
-			return fs.Rmdir(fdst, "")
+			return operations.Rmdir(fdst, "")
 		})
 	},
 }
