@@ -30,7 +30,7 @@ empty directories in.
 `,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
-		fdst := cmd.NewFsDst(args)
+		fdst := cmd.NewFsDir(args)
 		cmd.Run(true, false, command, func() error {
 			return operations.Rmdirs(fdst, "", leaveRoot)
 		})
