@@ -25,6 +25,8 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
+	"net/http"
+	"net/http/httptest"
 	"regexp"
 	"strings"
 	"testing"
@@ -96,6 +98,33 @@ func TestLs(t *testing.T) {
 	assert.Contains(t, res, "       60 potato2\n")
 }
 
+func TestLsWithFilesFrom(t *testing.T) {
+	r := fstest.NewRun(t)
+	defer r.Finalise()
+	file1 := r.WriteBoth("potato2", "------------------------------------------------------------", t1)
+	file2 := r.WriteBoth("empty space", "", t2)
+
+	fstest.CheckItems(t, r.Fremote, file1, file2)
+
+	// Set the --files-from equivalent
+	f, err := filter.NewFilter(nil)
+	require.NoError(t, err)
+	require.NoError(t, f.AddFile("potato2"))
+	require.NoError(t, f.AddFile("notfound"))
+
+	// Monkey patch the active filter
+	oldFilter := filter.Active
+	filter.Active = f
+	defer func() {
+		filter.Active = oldFilter
+	}()
+
+	var buf bytes.Buffer
+	err = operations.List(r.Fremote, &buf)
+	require.NoError(t, err)
+	assert.Equal(t, "       60 potato2\n", buf.String())
+}
+
 func TestLsLong(t *testing.T) {
 	r := fstest.NewRun(t)
 	defer r.Finalise()
@@ -238,14 +267,14 @@ func TestDelete(t *testing.T) {
 	fstest.CheckItems(t, r.Fremote, file3)
 }
 
-func testCheck(t *testing.T, checkFunction func(fdst, fsrc fs.Fs) error) {
+func testCheck(t *testing.T, checkFunction func(fdst, fsrc fs.Fs, oneway bool) error) {
 	r := fstest.NewRun(t)
 	defer r.Finalise()
 
-	check := func(i int, wantErrors int64) {
+	check := func(i int, wantErrors int64, oneway bool) {
 		fs.Debugf(r.Fremote, "%d: Starting check test", i)
 		oldErrors := accounting.Stats.GetErrors()
-		err := checkFunction(r.Flocal, r.Fremote)
+		err := checkFunction(r.Fremote, r.Flocal, oneway)
 		gotErrors := accounting.Stats.GetErrors() - oldErrors
 		if wantErrors == 0 && err != nil {
 			t.Errorf("%d: Got error when not expecting one: %v", i, err)
@@ -262,15 +291,15 @@ func testCheck(t *testing.T, checkFunction func(fdst, fsrc fs.Fs) error) {
 	file1 := r.WriteBoth("rutabaga", "is tasty", t3)
 	fstest.CheckItems(t, r.Fremote, file1)
 	fstest.CheckItems(t, r.Flocal, file1)
-	check(1, 0)
+	check(1, 0, false)
 
 	file2 := r.WriteFile("potato2", "------------------------------------------------------------", t1)
 	fstest.CheckItems(t, r.Flocal, file1, file2)
-	check(2, 1)
+	check(2, 1, false)
 
 	file3 := r.WriteObject("empty space", "", t2)
 	fstest.CheckItems(t, r.Fremote, file1, file3)
-	check(3, 2)
+	check(3, 2, false)
 
 	file2r := file2
 	if fs.Config.SizeOnly {
@@ -279,11 +308,16 @@ func testCheck(t *testing.T, checkFunction func(fdst, fsrc fs.Fs) error) {
 		r.WriteObject("potato2", "------------------------------------------------------------", t1)
 	}
 	fstest.CheckItems(t, r.Fremote, file1, file2r, file3)
-	check(4, 1)
+	check(4, 1, false)
 
 	r.WriteFile("empty space", "", t2)
 	fstest.CheckItems(t, r.Flocal, file1, file2, file3)
-	check(5, 0)
+	check(5, 0, false)
+
+	file4 := r.WriteObject("remotepotato", "------------------------------------------------------------", t1)
+	fstest.CheckItems(t, r.Fremote, file1, file2r, file3, file4)
+	check(6, 1, false)
+	check(7, 0, true)
 }
 
 func TestCheck(t *testing.T) {
@@ -369,6 +403,78 @@ func TestRcat(t *testing.T) {
 	check(false)
 }
 
+func TestPurge(t *testing.T) {
+	r := fstest.NewRunIndividual(t) // make new container (azureblob has delayed mkdir after rmdir)
+	defer r.Finalise()
+	r.Mkdir(r.Fremote)
+
+	// Make some files and dirs
+	r.ForceMkdir(r.Fremote)
+	file1 := r.WriteObject("A1/B1/C1/one", "aaa", t1)
+	//..and dirs we expect to delete
+	require.NoError(t, operations.Mkdir(r.Fremote, "A2"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A1/B2"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A1/B2/C2"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A1/B1/C3"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A3"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A3/B3"))
+	require.NoError(t, operations.Mkdir(r.Fremote, "A3/B3/C4"))
+	//..and one more file at the end
+	file2 := r.WriteObject("A1/two", "bbb", t2)
+
+	fstest.CheckListingWithPrecision(
+		t,
+		r.Fremote,
+		[]fstest.Item{
+			file1, file2,
+		},
+		[]string{
+			"A1",
+			"A1/B1",
+			"A1/B1/C1",
+			"A2",
+			"A1/B2",
+			"A1/B2/C2",
+			"A1/B1/C3",
+			"A3",
+			"A3/B3",
+			"A3/B3/C4",
+		},
+		fs.GetModifyWindow(r.Fremote),
+	)
+
+	require.NoError(t, operations.Purge(r.Fremote, "A1/B1"))
+
+	fstest.CheckListingWithPrecision(
+		t,
+		r.Fremote,
+		[]fstest.Item{
+			file2,
+		},
+		[]string{
+			"A1",
+			"A2",
+			"A1/B2",
+			"A1/B2/C2",
+			"A3",
+			"A3/B3",
+			"A3/B3/C4",
+		},
+		fs.GetModifyWindow(r.Fremote),
+	)
+
+	require.NoError(t, operations.Purge(r.Fremote, ""))
+
+	fstest.CheckListingWithPrecision(
+		t,
+		r.Fremote,
+		[]fstest.Item{},
+		[]string{},
+		fs.GetModifyWindow(r.Fremote),
+	)
+
+}
+
 func TestRmdirsNoLeaveRoot(t *testing.T) {
 	r := fstest.NewRun(t)
 	defer r.Finalise()
@@ -406,7 +512,29 @@ func TestRmdirsNoLeaveRoot(t *testing.T) {
 			"A3/B3",
 			"A3/B3/C4",
 		},
-		fs.Config.ModifyWindow,
+		fs.GetModifyWindow(r.Fremote),
+	)
+
+	require.NoError(t, operations.Rmdirs(r.Fremote, "A3/B3/C4", false))
+
+	fstest.CheckListingWithPrecision(
+		t,
+		r.Fremote,
+		[]fstest.Item{
+			file1, file2,
+		},
+		[]string{
+			"A1",
+			"A1/B1",
+			"A1/B1/C1",
+			"A2",
+			"A1/B2",
+			"A1/B2/C2",
+			"A1/B1/C3",
+			"A3",
+			"A3/B3",
+		},
+		fs.GetModifyWindow(r.Fremote),
 	)
 
 	require.NoError(t, operations.Rmdirs(r.Fremote, "", false))
@@ -422,7 +550,7 @@ func TestRmdirsNoLeaveRoot(t *testing.T) {
 			"A1/B1",
 			"A1/B1/C1",
 		},
-		fs.Config.ModifyWindow,
+		fs.GetModifyWindow(r.Fremote),
 	)
 
 }
@@ -447,7 +575,7 @@ func TestRmdirsLeaveRoot(t *testing.T) {
 			"A1/B1",
 			"A1/B1/C1",
 		},
-		fs.Config.ModifyWindow,
+		fs.GetModifyWindow(r.Fremote),
 	)
 
 	require.NoError(t, operations.Rmdirs(r.Fremote, "A1", true))
@@ -459,10 +587,58 @@ func TestRmdirsLeaveRoot(t *testing.T) {
 		[]string{
 			"A1",
 		},
-		fs.Config.ModifyWindow,
+		fs.GetModifyWindow(r.Fremote),
 	)
 }
 
+func TestRcatSize(t *testing.T) {
+	r := fstest.NewRun(t)
+	defer r.Finalise()
+
+	const body = "------------------------------------------------------------"
+	file1 := r.WriteFile("potato1", body, t1)
+	file2 := r.WriteFile("potato2", body, t2)
+	// Test with known length
+	bodyReader := ioutil.NopCloser(strings.NewReader(body))
+	obj, err := operations.RcatSize(r.Fremote, file1.Path, bodyReader, int64(len(body)), file1.ModTime)
+	require.NoError(t, err)
+	assert.Equal(t, int64(len(body)), obj.Size())
+	assert.Equal(t, file1.Path, obj.Remote())
+
+	// Test with unknown length
+	bodyReader = ioutil.NopCloser(strings.NewReader(body)) // reset Reader
+	ioutil.NopCloser(strings.NewReader(body))
+	obj, err = operations.RcatSize(r.Fremote, file2.Path, bodyReader, -1, file2.ModTime)
+	require.NoError(t, err)
+	assert.Equal(t, int64(len(body)), obj.Size())
+	assert.Equal(t, file2.Path, obj.Remote())
+
+	// Check files exist
+	fstest.CheckItems(t, r.Fremote, file1, file2)
+}
+
+func TestCopyURL(t *testing.T) {
+	r := fstest.NewRun(t)
+	defer r.Finalise()
+
+	contents := "file1 contents\n"
+	file1 := r.WriteFile("file1", contents, t1)
+	r.Mkdir(r.Fremote)
+	fstest.CheckItems(t, r.Fremote)
+
+	ts := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		_, err := w.Write([]byte(contents))
+		assert.NoError(t, err)
+	}))
+	defer ts.Close()
+
+	o, err := operations.CopyURL(r.Fremote, "file1", ts.URL)
+	require.NoError(t, err)
+	assert.Equal(t, int64(len(contents)), o.Size())
+
+	fstest.CheckListingWithPrecision(t, r.Fremote, []fstest.Item{file1}, nil, fs.ModTimeNotSupported)
+}
+
 func TestMoveFile(t *testing.T) {
 	r := fstest.NewRun(t)
 	defer r.Finalise()
@@ -694,34 +870,50 @@ func TestListFormat(t *testing.T) {
 	var list operations.ListFormat
 	list.AddPath()
 	list.SetDirSlash(false)
-	assert.Equal(t, "subdir", operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "subdir", list.Format(items[1]))
 
 	list.SetDirSlash(true)
-	assert.Equal(t, "subdir/", operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "subdir/", list.Format(items[1]))
 
 	list.SetOutput(nil)
-	assert.Equal(t, "", operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "", list.Format(items[1]))
 
 	list.AppendOutput(func() string { return "a" })
 	list.AppendOutput(func() string { return "b" })
-	assert.Equal(t, "ab", operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "ab", list.Format(items[1]))
 	list.SetSeparator(":::")
-	assert.Equal(t, "a:::b", operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "a:::b", list.Format(items[1]))
 
 	list.SetOutput(nil)
 	list.AddModTime()
-	assert.Equal(t, items[0].ModTime().Local().Format("2006-01-02 15:04:05"), operations.ListFormatted(&items[0], &list))
+	assert.Equal(t, items[0].ModTime().Local().Format("2006-01-02 15:04:05"), list.Format(items[0]))
+
+	list.SetOutput(nil)
+	list.AddID()
+	_ = list.Format(items[0]) // Can't really check anything - at least it didn't panic!
+
+	list.SetOutput(nil)
+	list.AddMimeType()
+	assert.Contains(t, list.Format(items[0]), "/")
+	assert.Equal(t, "inode/directory", list.Format(items[1]))
+
+	list.SetOutput(nil)
+	list.AddPath()
+	list.SetAbsolute(true)
+	assert.Equal(t, "/a", list.Format(items[0]))
+	list.SetAbsolute(false)
+	assert.Equal(t, "a", list.Format(items[0]))
 
 	list.SetOutput(nil)
 	list.AddSize()
-	assert.Equal(t, "1", operations.ListFormatted(&items[0], &list))
+	assert.Equal(t, "1", list.Format(items[0]))
 
 	list.AddPath()
 	list.AddModTime()
 	list.SetDirSlash(true)
 	list.SetSeparator("__SEP__")
-	assert.Equal(t, "1__SEP__a__SEP__"+items[0].ModTime().Local().Format("2006-01-02 15:04:05"), operations.ListFormatted(&items[0], &list))
-	assert.Equal(t, fmt.Sprintf("%d", items[1].Size())+"__SEP__subdir/__SEP__"+items[1].ModTime().Local().Format("2006-01-02 15:04:05"), operations.ListFormatted(&items[1], &list))
+	assert.Equal(t, "1__SEP__a__SEP__"+items[0].ModTime().Local().Format("2006-01-02 15:04:05"), list.Format(items[0]))
+	assert.Equal(t, fmt.Sprintf("%d", items[1].Size())+"__SEP__subdir/__SEP__"+items[1].ModTime().Local().Format("2006-01-02 15:04:05"), list.Format(items[1]))
 
 	for _, test := range []struct {
 		ht   hash.Type
@@ -733,9 +925,20 @@ func TestListFormat(t *testing.T) {
 	} {
 		list.SetOutput(nil)
 		list.AddHash(test.ht)
-		got := operations.ListFormatted(&items[0], &list)
+		got := list.Format(items[0])
 		if got != "UNSUPPORTED" && got != "" {
 			assert.Equal(t, test.want, got)
 		}
 	}
+
+	list.SetOutput(nil)
+	list.SetSeparator("|")
+	list.SetCSV(true)
+	list.AddSize()
+	list.AddPath()
+	list.AddModTime()
+	list.SetDirSlash(true)
+	assert.Equal(t, "1|a|"+items[0].ModTime().Local().Format("2006-01-02 15:04:05"), list.Format(items[0]))
+	assert.Equal(t, fmt.Sprintf("%d", items[1].Size())+"|subdir/|"+items[1].ModTime().Local().Format("2006-01-02 15:04:05"), list.Format(items[1]))
+
 }
